---
title: Server API | ldapjs
markdown2extras: wiki-tables
logo-color: green
logo-font-family: google:Aldrich, Verdana, sans-serif
header-font-family: google:Aldrich, Verdana, sans-serif
---

# Backend Configuration and Tuning

This document describes how to create an use a ldapjs-riak backend,
tuning, and setting up Riak.

# Install Riak

Obviously, to leverage Riak, you need to install Riak.  Grab a 1.0.x
release from [Basho](http://basho.com), and follow their setup
instructions.  Post-install, you'll need to edit Riak's `app.config`
`storage_backend` setting to:

    {storage_backend, riak_kv_eleveldb_backend},

The default will have been `bitcask`.  ldapjs-riak basically doesn't
work, at all, without Riak's 2i feature, so this is required.

Other than that, do whatever you would do with Riak to setup a
cluster, tune memory setttings, add a load balancer, etc.  It's out of
scope for this document to tell you how to deploy Riak to production...

# What can I configure?

The Riak backend has the following configurations:

- Cluster information
- CAP tuning
- Indexes/Unique Indexes
- Changelog

Each of these is now explained.

## Riak Cluster


# Creating a backend suitable for ldapjs

If you're not familiar wth [ldapjs](http://ldapjs.org), get familiar,
as the rest of this won't make any sense otherwise.  ldapjs includes
the ability to keep a "backend" object that is stateful, and this
module leverages that functionality.  The bare minimum you need to get
going is the following:

    var ldapRiak = require('ldapjs-riak');
    var backend = ldapRiak.createBackend({
        "bucket": {
          "name": "ldapjs_riak",
        },
        "uniqueIndexBucket": {
          "name": ldapjs_riak_uindex",
        },
        "client": {
          "url": "http://localhost:8098",
          "clientId": "ldapjs_riak"
        }
    });

Which will create a backend, and point it at the specified Riak
host/port/buckets, with no indexes.  Once you have that, you can mount
the backend "as normal" in ldapjs:

    var ldap = require('ldapjs');

    var SUFFIX = 'dc=example, dc=com';

    var server = ldap.createServer({});

    server.add(SUFFIX, backend, backend.add());
    server.modify(SUFFIX, backend, backend.modify());
    server.bind(SUFFIX, backend, backend.bind());
    server.compare(SUFFIX, backend, backend.compare());
    server.del(SUFFIX, backend, backend.del());
    server.modifyDN(SUFFIX, backend, backend.modifyDN());
    server.search(SUFFIX, backend, backend.search());

While that's kind of annoyingly verbose, each of the operations takes
the ability to inject handlers that run after backend
intiialization has been run, but before "real work" gets kicked
off. So for example:

    server.compare(SUFFIX, backend, function(req, res, next) {
      return next();
    }, backend.compare(function(req, res, next) {
      req.riak.log('hello world');
    }));

While that does nothing interesting, it does show that you can still
use "normal" handlers with ldapjs, as well as special "ldapjs-riak" handlers.

# createBackend(options)

The full list of options (options is a plain JS object) to `createBackend` is:

||bucket||Object||required||A configuration of the Riak bucket name and CAP tunings for entries. ||
||log4js||Log4JS Instance||required||`require('log4js')` or other configured instance.||
||client||Object||required||Connection information for the actual Riak cluster.||
||uniqueIndexBucket||Object||optional||A configuration of the Riak bucket name and CAP tunings for unique indexes.||
||changelogBucket||Object||optional||A configuration of the Riak bucket name and CAP tunings for changelogging.||
||indexes||Object||optional||A listing of attributes to index in an entry, and whether or not uniquness should be enforced.||

